package net.sf.corn.gate;

import net.sf.corn.exception.ExceptionBuilder;
import net.sf.corn.exception.ExceptionCommand;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;



/**
 * Service Locator implementation for Plain Old Java Objects.This implementation is 
 * used as default service locator by the gate. Service definitions that use 
 * this locator must put class names as service targets. Because of this 
 * locator defined as default locator, services that uses this locator doesn't have
 * to put "locator" attribute in their service definitions.
 * 
 * @author Serhat Dirik
 */
public class POJOLocator implements ILocator {
	private static Logger log = LoggerFactory.getLogger(POJOLocator.class);
	private static ExceptionCommand cmdLogError = new ExceptionCommand() {
		
		@Override
		public void execute(Exception e) {
			if(log.isDebugEnabled()) log.error(e.getMessage(),e);
		}
	};
	public static final String NAME = ILocator.DEFAULT_LOCATOR_NAME;
	
	@Override
	public Object getComponentInstance(ServiceDefinition serviceDef)
	throws GateException {
		try {
			Object service = Class.forName(serviceDef.getComponentTarget()).newInstance();
			return service;
		} catch (Exception e) {
			throw ExceptionBuilder.with(GateException.class).cause(e).fill("Component target class %1! component couldn't initialized!", serviceDef.getComponentTarget()).
				execute(cmdLogError).build().code(GateException.CODE_COMPONENT_INITIALIZATION);
		}
	}
	
	@Override
	public String getName() {
		return ILocator.DEFAULT_LOCATOR_NAME;
	}
	@Override
	public String toString() {
		return getName()+" Locator";
	}
	
}
